import { WidgetType } from "@codemirror/view";
import { useMemo, useState } from "react";
// a CodeMirror widget which reports to a PortalSet for React rendering
export class PortalWidget extends WidgetType {
    constructor(portalSet, data) {
        super();
        this.portalSet = portalSet;
        this.data = data;
    }
    eq(otherWidget) {
        return this.portalSet === otherWidget.portalSet;
    }
    toDOM() {
        this.elem = document.createElement("span");
        this.portalSet.register(this.elem, this.data);
        return this.elem;
    }
    destroy() {
        this.elem && this.portalSet.unregister(this.elem);
    }
}
// a generic tool for maintaining sets of React portals coming out of an imperative system
// (like CodeMirror!)
export class PortalSet {
    constructor(setPortals) {
        this.setPortals = setPortals;
        this.portalMap = new Map();
    }
    portals() {
        return Array.from(this.portalMap.entries());
    }
    register(elem, data) {
        this.portalMap.set(elem, data);
        this.setPortals(this.portals());
    }
    unregister(elem) {
        this.portalMap.delete(elem);
        this.setPortals(this.portals());
    }
}
export function usePortalSet() {
    const [portals, setPortals] = useState([]);
    const portalSet = useMemo(() => new PortalSet(setPortals), []);
    return [portalSet, portals];
}
//# sourceMappingURL=PortalWidget.js.map